Stan Math Library: stan/math/prim/scal/prob/cauchy_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

cauchy_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_CAUCHY_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_CAUCHY_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/square.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/log1p.hpp>
15 #include <stan/math/prim/scal/meta/include_summand.hpp>
16 #include <boost/random/cauchy_distribution.hpp>
17 #include <boost/random/variate_generator.hpp>
18 #include <cmath>
19
20 namespace stan {
21
22 namespace math {
23
24 template <typename T_y, typename T_loc, typename T_scale>
25 typename return_type<T_y, T_loc, T_scale>::type
26 cauchy_cdf_log(const T_y& y, const T_loc& mu, const T_scale& sigma) {
27 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
28 T_partials_return;
29
30 // Size checks
31 if ( !( stan::length(y) && stan::length(mu)
32 && stan::length(sigma) ) )
33 return 0.0;
34
35 static const char* function("stan::math::cauchy_cdf");
36
37 using stan::math::check_positive_finite;
38 using stan::math::check_finite;
39 using stan::math::check_not_nan;
40 using stan::math::check_consistent_sizes;
41 using boost::math::tools::promote_args;
42 using stan::math::value_of;
43
44 T_partials_return cdf_log(0.0);
45
46 check_not_nan(function, "Random variable", y);
47 check_finite(function, "Location parameter", mu);
48 check_positive_finite(function, "Scale parameter", sigma);
49 check_consistent_sizes(function,
50 "Random variable", y,
51 "Location parameter", mu,
52 "Scale Parameter", sigma);
53
54 // Wrap arguments in vectors
55 VectorView<const T_y> y_vec(y);
56 VectorView<const T_loc> mu_vec(mu);
57 VectorView<const T_scale> sigma_vec(sigma);
58 size_t N = max_size(y, mu, sigma);
59
60 OperandsAndPartials<T_y, T_loc, T_scale>
61 operands_and_partials(y, mu, sigma);
62
63 // Compute CDFLog and its gradients
64 using std::atan;
65 using stan::math::pi;
66 using std::log;
67
68 // Compute vectorized CDF and gradient
69 for (size_t n = 0; n < N; n++) {
70 // Pull out values
71 const T_partials_return y_dbl = value_of(y_vec[n]);
72 const T_partials_return mu_dbl = value_of(mu_vec[n]);
73 const T_partials_return sigma_inv_dbl = 1.0 / value_of(sigma_vec[n]);
74 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
75
76 const T_partials_return z = (y_dbl - mu_dbl) * sigma_inv_dbl;
77
78 // Compute
79 const T_partials_return Pn = atan(z) / pi() + 0.5;
80 cdf_log += log(Pn);
81
82 const T_partials_return rep_deriv
83 = 1.0 / (pi() * Pn * (z * z * sigma_dbl + sigma_dbl));
84 if (!is_constant_struct<T_y>::value)
85 operands_and_partials.d_x1[n] += rep_deriv;
86 if (!is_constant_struct<T_loc>::value)
87 operands_and_partials.d_x2[n] -= rep_deriv;
88 if (!is_constant_struct<T_scale>::value)
89 operands_and_partials.d_x3[n] -= rep_deriv * z;
90 }
91 return operands_and_partials.value(cdf_log);
92 }
93
94 }
95 }
96 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::cauchy_cdf_log
return_type< T_y, T_loc, T_scale >::type cauchy_cdf_log(const T_y &y, const T_loc &mu, const T_scale &sigma)
Definition: cauchy_cdf_log.hpp:26
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
square.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::atan
fvar< T > atan(const fvar< T > &x)
Definition: atan.hpp:12
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
log1p.hpp
stan::math::pi
double pi()
Return the value of pi.
Definition: constants.hpp:86
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.